# -*- coding: utf-8 -*-

CPUQuotaStatus = int

# CPUQuotaUndefined is returned when CPU quota is undefined
CPUQuotaUndefined = 0
# CPUQuotaUsed is returned when a valid CPU quota can be used
CPUQuotaUsed = 1
# CPUQuotaMinUsed is return when CPU quota is smaller than the min value
CPUQuotaMinUsed = 2
